@@ -38,7 +38,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.9.2.10 2002/04/24 01:49:34 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.9.2.18 2002/11/20 03:18:47 zacheiss Exp $");
 
 #include "../afs/sysincludes.h" /*Standard vendor system headers*/
 #include "../afs/afsincludes.h" /*AFS-based standard headers*/
@@ -70,7 +70,7 @@ extern struct vcache *afs_globalVp;
 #ifdef AFS_OSF_ENV
 extern struct mount *afs_globalVFS;
 extern struct vnodeops Afs_vnodeops;
-#elif defined(AFS_DARWIN_ENV)
+#elif defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 extern struct mount *afs_globalVFS;
 #else
 extern struct vfs *afs_globalVFS;
@@ -467,6 +467,79 @@ static afs_int32 afs_QueueVCB(struct vcache *avc)
 }
 
 #ifdef AFS_LINUX22_ENV
+
+static void __shrink_dcache_parent(struct dentry * parent)
+{
+	struct dentry *this_parent = parent;
+	struct list_head *next;
+	int found = 0;
+	LIST_HEAD(afs_dentry_unused);
+
+repeat:
+	next = this_parent->d_subdirs.next;
+resume:
+	while (next != &this_parent->d_subdirs) {
+		struct list_head *tmp = next;
+		struct dentry *dentry = list_entry(tmp, struct dentry, d_child);
+		next = tmp->next;
+		if (!DCOUNT(dentry)) {
+			list_del(&dentry->d_lru);
+			list_add(&dentry->d_lru, afs_dentry_unused.prev);
+			found++;
+		}
+		/*
+		 * Descend a level if the d_subdirs list is non-empty.
+		 */
+		if (!list_empty(&dentry->d_subdirs)) {
+			this_parent = dentry;
+			goto repeat;
+		}
+	}
+	/*
+	 * All done at this level ... ascend and resume the search.
+	 */
+	if (this_parent != parent) {
+		next = this_parent->d_child.next; 
+		this_parent = this_parent->d_parent;
+		goto resume;
+	}
+
+	for (;;) {
+		struct dentry *dentry;
+		struct list_head *tmp;
+
+		tmp = afs_dentry_unused.prev;
+
+		if (tmp == &afs_dentry_unused)
+			break;
+#ifdef AFS_LINUX24_ENV
+		list_del_init(tmp);
+#else
+		list_del(tmp);
+		INIT_LIST_HEAD(tmp);
+#endif /* AFS_LINUX24_ENV */
+		dentry = list_entry(tmp, struct dentry, d_lru);
+
+#ifdef AFS_LINUX24_ENV
+		/* Unused dentry with a count? */
+		if (DCOUNT(dentry))
+			BUG();
+#endif
+		DGET(dentry);
+#ifdef AFS_LINUX24_ENV
+		list_del_init(&dentry->d_hash);		/* d_drop */
+#else
+		list_del(&dentry->d_hash);
+		INIT_LIST_HEAD(&dentry->d_hash);
+#endif /* AFS_LINUX24_ENV */
+		DUNLOCK();
+		dput(dentry);
+		DLOCK();
+		if (!--found)
+			break;
+	}
+}
+
 /* afs_TryFlushDcacheChildren -- Shakes loose vcache references held by
  *                               children of the dentry
  *
@@ -485,49 +558,48 @@ static void afs_TryFlushDcacheChildren(struct vcache *tvc)
     struct list_head *head = &ip->i_dentry;
     struct dentry *dentry;
     
+    AFS_GUNLOCK();
 restart:
+#ifndef old_vcache_scheme
     DLOCK();
     cur = head;
     while ((cur = cur->next) != head) {
 	dentry = list_entry(cur, struct dentry, d_alias);
-	if (DCOUNT(dentry)) {
-	    this_parent = dentry;
-	repeat:
-	    next = this_parent->d_subdirs.next;
-	resume:
-	    while (next != &this_parent->d_subdirs) {
-		struct list_head *tmp = next;
-		struct dentry *dchld = list_entry(tmp, struct dentry, d_child);
-		
-		next = tmp->next;
-		if (!DCOUNT(dchld) && !dchld->d_inode) {
-		    DGET(dchld);
-		    AFS_GUNLOCK();
-		    DUNLOCK();
-		    d_drop(dchld);
-		    dput(dchld);
-		    AFS_GLOCK();
-		    DLOCK();
-		    goto repeat;
-		}
-		/*
-		 * Descend a level if the d_subdirs list is non-empty.
-		 */
-		if (!list_empty(&dchld->d_subdirs)) {
-		    this_parent = dchld;
-		    goto repeat;
-		}
-	    }
-	    
-	    /*
-	     * All done at this level ... ascend and resume the search.
-	     */
-	    if (this_parent != dentry) {
-		next = this_parent->d_child.next;
-		this_parent = this_parent->d_parent;
-		goto resume;
-	    }
-	}
+
+	afs_Trace3(afs_iclSetp, CM_TRACE_TRYFLUSHDCACHECHILDREN,
+		   ICL_TYPE_POINTER, ip,
+		   ICL_TYPE_STRING, dentry->d_parent->d_name.name,
+		   ICL_TYPE_STRING, dentry->d_name.name);
+
+        if (!list_empty(&dentry->d_hash) && !list_empty(&dentry->d_subdirs))
+            __shrink_dcache_parent(dentry);
+
+        if (!DCOUNT(dentry)) {
+            DGET(dentry);
+#ifdef AFS_LINUX24_ENV 
+            list_del_init(&dentry->d_hash);     /* d_drop */
+#else
+	    list_del(&dentry->d_hash);
+	    INIT_LIST_HEAD(&dentry->d_hash);
+#endif /* AFS_LINUX24_ENV */
+            DUNLOCK();
+            dput(dentry);
+            goto restart;
+        }
+    }
+    DUNLOCK();
+    AFS_GLOCK();
+#else
+restart:
+    DLOCK();
+    cur = head;
+    while ((cur = cur->next) != head) {
+        dentry = list_entry(cur, struct dentry, d_alias);
+
+        afs_Trace3(afs_iclSetp, CM_TRACE_TRYFLUSHDCACHECHILDREN,
+                   ICL_TYPE_POINTER, ip,
+                   ICL_TYPE_STRING, dentry->d_parent->d_name.name,
+                   ICL_TYPE_STRING, dentry->d_name.name);
 
 	if (!DCOUNT(dentry)) {
 	    AFS_GUNLOCK();
@@ -540,7 +612,7 @@ restart:
 	}
     }
     DUNLOCK();
-
+#endif
 }
 #endif /* AFS_LINUX22_ENV */
 
@@ -907,6 +979,14 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
     tvc->v.v_freelist.tqe_prev=(struct vnode **)0xdeadb;
     /*tvc->vrefCount++;*/
 #endif 
+#ifdef AFS_FBSD_ENV
+    lockinit(&tvc->rwlock, PINOD, "vcache rwlock", 0, 0);
+    cache_purge(AFSTOV(tvc)); 
+    tvc->v.v_data=tvc;
+    tvc->v.v_tag=VT_AFS;
+    tvc->v.v_usecount++; /* steal an extra ref for now so vfree never happens */
+                         /* This extra ref is dealt with above... */
+#endif
     /*
      * The proper value for mvstat (for root fids) is setup by the caller.
      */
@@ -977,6 +1057,9 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 	INIT_LIST_HEAD(&ip->i_devices);
 #endif
 	ip->i_data.host = (void*) ip;
+#ifdef STRUCT_ADDRESS_SPACE_HAS_GFP_MASK
+	ip->i_data.gfp_mask = GFP_HIGHUSER;
+#endif
 	ip->i_mapping = &ip->i_data;
 #ifdef STRUCT_INODE_HAS_I_TRUNCATE_SEM
 	init_rwsem(&ip->i_truncate_sem);
@@ -1486,9 +1569,16 @@ afs_ProcessFS(avc, astat, areq)
 	avc->m.Mode |= S_IFDIR;
     }
     else if (astat->FileType == SymbolicLink) {
-	vSetType(avc, VLNK);
-	avc->m.Mode |= S_IFLNK;
-	if ((avc->m.Mode & 0111) == 0) avc->mvstat = 1;
+	if (afs_fakestat_enable && (avc->m.Mode & 0111) == 0) {
+	    vSetType(avc, VDIR);
+	    avc->m.Mode |= S_IFDIR;
+	} else {
+	    vSetType(avc, VLNK);
+	    avc->m.Mode |= S_IFLNK;
+	}
+	if ((avc->m.Mode & 0111) == 0) {
+	    avc->mvstat = 1;
+	}
     }
     avc->anyAccess = astat->AnonymousAccess;
 #ifdef badidea
@@ -1519,6 +1609,9 @@ afs_ProcessFS(avc, astat, areq)
 #ifdef AFS_LINUX22_ENV
     vcache2inode(avc);    /* Set the inode attr cache */
 #endif
+#ifdef AFS_DARWIN_ENV
+   osi_VM_Setup(avc,1);
+#endif
 
 } /*afs_ProcessFS*/
 
@@ -1660,7 +1753,7 @@ loop:
 #endif
 	ReleaseWriteLock(&tvc->lock);
 #ifdef AFS_DARWIN_ENV
-        osi_VM_Setup(tvc);
+        osi_VM_Setup(tvc,0);
 #endif
 	return tvc;
     }
@@ -1725,9 +1818,6 @@ loop:
     }
 
     ReleaseWriteLock(&tvc->lock);
-#ifdef AFS_DARWIN_ENV
-    osi_VM_Setup(avc);
-#endif
     return tvc;
 
 } /*afs_GetVCache*/
@@ -1883,9 +1973,6 @@ struct vcache *afs_LookupVCache(struct VenusFid *afid, struct vrequest *areq,
     afs_ProcessFS(tvc, &OutStatus, areq);
 
     ReleaseWriteLock(&tvc->lock);
-#ifdef AFS_DARWIN_ENV
-    osi_VM_Setup(tvc);
-#endif
     return tvc;
 
 }
@@ -2455,7 +2542,7 @@ struct vcache *afs_FindVCache(struct VenusFid *afid, afs_int32 lockit,
 #endif
 #ifdef AFS_DARWIN_ENV
     if (tvc)
-        osi_VM_Setup(tvc);
+        osi_VM_Setup(tvc, 0);
 #endif
     return tvc;
 } /*afs_FindVCache*/
