@@ -23,12 +23,140 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/rx/FBSD/rx_knet.c,v 1.5 2001/08/08 06:29:02 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/rx/FBSD/rx_knet.c,v 1.5.2.1 2002/09/10 22:33:57 zacheiss Exp $");
 
 #ifdef AFS_FBSD40_ENV
 #include "../rx/rx_kcommon.h"
 
 
+#ifdef RXK_LISTENER_ENV
+int osi_NetReceive(osi_socket so, struct sockaddr_in *addr, struct iovec *dvec,         
+        int nvecs, int *alength)
+{   
+    struct socket *asocket = (struct socket *)so;
+    struct uio u;
+    int i;
+    struct iovec iov[RX_MAXIOVECS];
+    struct sockaddr *sa;
+    int code;
+
+    int haveGlock = ISAFS_GLOCK();
+    /*AFS_STATCNT(osi_NetReceive);*/
+
+    if (nvecs > RX_MAXIOVECS) {
+        osi_Panic("osi_NetReceive: %d: Too many iovecs.\n", nvecs);
+    }
+
+    for (i = 0 ; i < nvecs ; i++) {
+        iov[i].iov_base = dvec[i].iov_base;
+        iov[i].iov_len = dvec[i].iov_len;
+    }
+
+    u.uio_iov=&iov[0];
+    u.uio_iovcnt=nvecs;
+    u.uio_offset=0;
+    u.uio_resid=*alength;
+    u.uio_segflg=UIO_SYSSPACE; 
+    u.uio_rw=UIO_READ;
+    u.uio_procp=NULL;  
+
+    if (haveGlock) { 
+        AFS_GUNLOCK();
+    }
+    code = soreceive(asocket, &sa, &u, NULL, NULL, NULL);
+#if KNET_DEBUG
+    if (code) {
+        if (code == EINVAL)
+          Debugger("afs NetReceive busted");
+        else
+          printf("y");
+    }
+#endif
+    if (haveGlock) {
+        AFS_GLOCK();
+    }
+    *alength=*alength-u.uio_resid;
+    if (sa) {
+       if (sa->sa_family == AF_INET) {
+          if (addr) *addr=*(struct sockaddr_in *)sa;
+       } else {
+          printf("Unknown socket family %d in NetReceive\n", sa->sa_family);
+       }
+    }
+    return code;
+}
+
+extern int rxk_ListenerPid;
+void osi_StopListener(void)
+{
+   struct proc *p;
+
+   soclose(rx_socket);
+   p=pfind(rxk_ListenerPid); 
+   if (p)
+       psignal(p, SIGUSR1);
+}
+
+int 
+osi_NetSend(asocket, addr, dvec, nvecs, alength, istack)
+     register struct socket *asocket;
+     struct iovec *dvec;
+     int nvecs;
+     register afs_int32 alength;
+     struct sockaddr_in *addr;
+     int istack;
+{
+    register afs_int32 code;
+    int s;
+    int len;
+    int i;
+    struct iovec iov[RX_MAXIOVECS];
+    char *tdata;
+    struct uio u;
+    int haveGlock = ISAFS_GLOCK();
+
+    AFS_STATCNT(osi_NetSend);
+    if (nvecs > RX_MAXIOVECS) {
+        osi_Panic("osi_NetSend: %d: Too many iovecs.\n", nvecs);
+    } 
+
+    for (i = 0 ; i < nvecs ; i++) {
+        iov[i].iov_base = dvec[i].iov_base; 
+        iov[i].iov_len = dvec[i].iov_len; 
+    } 
+
+    u.uio_iov=&iov[0];
+    u.uio_iovcnt=nvecs;
+    u.uio_offset=0;
+    u.uio_resid=alength;
+    u.uio_segflg=UIO_SYSSPACE;
+    u.uio_rw=UIO_WRITE;
+    u.uio_procp=NULL;
+
+    addr->sin_len=sizeof(struct sockaddr_in);
+
+    if (haveGlock) {
+        AFS_GUNLOCK();
+    }
+#if KNET_DEBUG
+    printf("+");
+#endif
+    code = sosend(asocket, addr, &u, NULL, NULL, 0, curproc);
+#if KNET_DEBUG
+    if (code) {
+        if (code == EINVAL)
+          Debugger("afs NetSend busted");
+        else
+          printf("z");
+    }
+#endif
+    if (haveGlock) {
+        AFS_GLOCK();
+    }
+    return code;
+}
+#else
+/* This code *almost* works :( */
 static struct protosw parent_proto;	/* udp proto switch */
 static void rxk_input (struct mbuf *am, int iphlen);
 static void rxk_fasttimo (void);
@@ -41,9 +169,11 @@ rxk_init() {
     last = inetdomain.dom_protoswNPROTOSW;
     for (tpro = inetdomain.dom_protosw; tpro < last; tpro++)
       if (tpro->pr_protocol == IPPROTO_UDP) {
+#if 0 /* not exported */
 	/* force UDP checksumming on for AFS	*/
 	 extern int udpcksum;
 	 udpcksum = 1;	
+#endif
           memcpy(&parent_proto, tpro, sizeof(parent_proto));
           tpro->pr_input = rxk_input;
           tpro->pr_fasttimo = rxk_fasttimo;
@@ -190,7 +320,6 @@ static void rxk_fasttimo (void)
     if (tproc = parent_proto.pr_fasttimo) (*tproc)();
 }
 
-
 /* rx_NetSend - send asize bytes at adata from asocket to host at addr.
  *
  * Now, why do we allocate a new buffer when we could theoretically use the one
@@ -219,6 +348,8 @@ register struct sockbuf *sb; {
     return 0;
 }
 
+/* We only have to do all the mbuf management ourselves if we can be called at
+   interrupt time. in RXK_LISTENER_ENV, we can just call sosend() */
 int 
 osi_NetSend(asocket, addr, dvec, nvec, asize, istack)
      register struct socket *asocket;
@@ -239,15 +370,20 @@ osi_NetSend(asocket, addr, dvec, nvec, asize, istack)
     int i,tl,rlen;
     int mlen;
     int haveGlock;
+#if KNET_DEBUG
+    static int before=0;
+#endif
 
     AFS_STATCNT(osi_NetSend);
-
 /* Actually, the Ultrix way is as good as any for us, so we don't bother with
  * special mbufs any more.  Used to think we could get away with not copying
  * the data to the interface, but there's no way to tell the caller not to
  * reuse the buffers after sending, so we lost out on that trick anyway */
-
     s = splnet();
+    if (trysblock(&asocket->so_snd)) {
+        splx(s);
+        return 1;
+    }
     mp = &top;
     i = 0;
     tdata = dvec[i].iov_base;
@@ -257,6 +393,7 @@ osi_NetSend(asocket, addr, dvec, nvec, asize, istack)
         if (top == 0) {
             MGETHDR(m, M_DONTWAIT, MT_DATA);
             if (!m) {
+                sbunlock(&asocket->so_snd);
                 splx(s);
                 return 1;
             }
@@ -282,8 +419,6 @@ osi_NetSend(asocket, addr, dvec, nvec, asize, istack)
 	 * to ourself).
 	 */
 	if (asize >= 4 * MLEN) {	/* try to get cluster mbuf */
-	    register struct mbuf *p;
-
 	    /* different algorithms for getting cluster mbuf */
             MCLGET(m, M_DONTWAIT);
             if ((m->m_flags & M_EXT) == 0)
@@ -334,32 +469,42 @@ nopages:
     um = m_get(M_DONTWAIT, MT_SONAME);
     if (!um) {
 	if (top) m_freem(top);	/* free mbuf chain */
-	/* if this were vfs40, we'd do sbunlock(asocket, &asocket->so_snd), but
-	   we don't do the locking at all for vfs40 systems */
+        sbunlock(&asocket->so_snd);
 	splx(s);
 	return 1;
     }
     memcpy(mtod(um, caddr_t), addr, sizeof(*addr));
-    um->m_len = sizeof(*addr);
+    addr->sin_len = um->m_len = sizeof(*addr);
     /* note that udp_usrreq frees funny mbuf.  We hold onto data, but mbuf
-     * around it is gone.  we free address ourselves.  */
+     * around it is gone. */
     /*    haveGlock = ISAFS_GLOCK();
     if (haveGlock) {
 	AFS_GUNLOCK();
 	}  */
     /* SOCKET_LOCK(asocket); */
     /* code = (*asocket->so_proto->pr_usrreq)(asocket, PRU_SEND, tm, um, 0); */
+#if KNET_DEBUG
+    if (before) Debugger("afs NetSend before");
+#endif
     code = (*asocket->so_proto->pr_usrreqs->pru_send)(asocket, 0, tm, 
 						      (struct sockaddr *) addr,
-						      um, curproc);
+						      um, &proc0);
     /* SOCKET_UNLOCK(asocket); */
     /* if (haveGlock) {
 	AFS_GLOCK();
 	} */
+    sbunlock(&asocket->so_snd);
     splx(s);
-    m_free(um);
-
+#if KNET_DEBUG
+    if (code) {
+        if (code == EINVAL)
+          Debugger("afs NetSend busted");
+        else
+          printf("z");
+    }
+#endif
     return code;
 }
+#endif
 
 #endif /* AFS_FBSD40_ENV */
