@@ -14,7 +14,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/rxkad/rxkad_server.c,v 1.7 2001/08/08 00:04:05 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/rxkad/rxkad_server.c,v 1.7.2.3 2002/11/14 22:44:38 zacheiss Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -24,6 +24,13 @@ RCSID("$Header: /cvs/openafs/src/rxkad/rxkad_server.c,v 1.7 2001/08/08 00:04:05
 #else
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <des.h>
@@ -180,7 +187,6 @@ rxs_return_t rxkad_GetChallenge (aobj, aconn, apacket)
   IN struct rx_packet *apacket;
   IN struct rx_connection *aconn;
 {   struct rxkad_sconn *sconn;
-    afs_int32 temp;
     char *challenge;
     int challengeSize;
     struct rxkad_v2Challenge  c_v2;   /* version 2 */
@@ -298,6 +304,23 @@ rxs_return_t rxkad_CheckResponse (aobj, aconn, apacket)
      * If the alternate decoder is not present, or returns -1, then
      * assume the ticket is of the default style.
      */
+    if (code == -1 && 
+	(kvno == RXKAD_TKT_TYPE_KERBEROS_V5) ||
+	(kvno == RXKAD_TKT_TYPE_KERBEROS_V5_ENCPART_ONLY)) 
+    {
+	code = tkt_DecodeTicket5 (tix, tlen, 
+				  tsp->get_key,
+				  tsp->get_key_rock,
+				  kvno,
+				  client.name, client.instance, client.cell,
+				  &sessionkey, &host, &start, &end);
+	if (code) return RXKADBADTICKET;
+    }
+
+    /*
+     * If the alternate decoder/kerberos 5 decoder is not present, or
+     * returns -1, then assume the ticket is of the default style.
+     */
     if (code == -1) {
 	/* get ticket's key */
 	code = (*tsp->get_key)(tsp->get_key_rock, kvno, &serverKey);
