@@ -82,7 +82,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6.2.1 2001/10/13 04:22:10 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6.2.5 2002/11/15 04:07:58 zacheiss Exp $");
 
 #include <stdio.h> 
 #include <stdlib.h>      /* for malloc() */
@@ -1391,16 +1391,16 @@ static int lih_r(host, held, hostp)
 
 } /*lih*/
 
-
 /* This could be upgraded to get more space each time */
 /* first pass: find the oldest host which isn't held by anyone */
 /* second pass: find the oldest host who isn't "me" */
 /* always called with hostp unlocked */
+extern struct host *hostList;
 static int GetSomeSpace_r(hostp, locked)
     struct host *hostp;
     int locked;
 {
-    register struct host *hp, *hp1 = (struct host *)0;
+    register struct host *hp, *hp1 = (struct host *)0, *hp2 = hostList;
     int i=0;
 
     cbstuff.GotSomeSpaces++;
@@ -1411,13 +1411,15 @@ static int GetSomeSpace_r(hostp, locked)
     }
     do {
 	lih_host = 0;
-	h_Enumerate_r(lih_r, (char *)hp1);
+	h_Enumerate_r(lih_r, hp2, (char *)hp1);
 	hp = lih_host;
 	if (hp) {
 	    cbstuff.GSS4++;
 	    if ( ! ClearHostCallbacks_r(hp, 0 /* not locked or held */) )
 		return;
+	    hp2 = hp->next;
 	} else {
+	    hp2 = hostList;
 	    hp1 = hostp;
 	    cbstuff.GSS1++;
 	    ViceLog(5,("GSS: Try harder for longest inactive host cnt= %d\n", i));
@@ -1789,9 +1791,9 @@ struct host*		host;
 struct AFSCBFids*	afidp;
 {
 	int i,j;
-	struct rx_connection*	conns[AFS_MAX_INTERFACE_ADDR];
+	struct rx_connection**	conns;
 	struct rx_connection*   connSuccess = 0;
-	afs_int32			addr[AFS_MAX_INTERFACE_ADDR];
+	afs_int32		*addr;
 	static struct rx_securityClass *sc = 0;
 	static struct AFSCBs tc = {0,0};
 
@@ -1808,6 +1810,10 @@ struct AFSCBFids*	afidp;
 	if ( !sc )
 	    sc = (struct rx_securityClass *) rxnull_NewClientSecurityObject();
 
+	i = host->interface->numberOfInterfaces;
+	addr = malloc(i * sizeof(afs_int32));
+	conns = malloc(i * sizeof(struct rx_connection *));
+
 	/* initialize alternate rx connections */
 	for ( i=0,j=0; i < host->interface->numberOfInterfaces; i++)
 	{
@@ -1854,6 +1860,9 @@ struct AFSCBFids*	afidp;
 		if ( conns[i] != connSuccess )
 			rx_DestroyConnection(conns[i] );
 
+	free(addr);
+	free(conns);
+
 	if ( connSuccess ) return 0;	/* success */
 		else return 1;		/* failure */
 }
@@ -1868,9 +1877,9 @@ MultiProbeAlternateAddress_r(host)
 struct host*		host;
 {
 	int i,j;
-	struct rx_connection*	conns[AFS_MAX_INTERFACE_ADDR];
+	struct rx_connection**	conns;
 	struct rx_connection*   connSuccess = 0;
-	afs_int32			addr[AFS_MAX_INTERFACE_ADDR];
+	afs_int32		*addr;
 	static struct rx_securityClass *sc = 0;
 
 	/* nothing more can be done */
@@ -1886,6 +1895,10 @@ struct host*		host;
 	if ( !sc )
 	    sc = (struct rx_securityClass *) rxnull_NewClientSecurityObject();
 
+	i = host->interface->numberOfInterfaces;
+	addr = malloc(i * sizeof(afs_int32));
+	conns = malloc(i * sizeof(struct rx_connection *));
+
 	/* initialize alternate rx connections */
 	for ( i=0,j=0; i < host->interface->numberOfInterfaces; i++)
 	{
@@ -1932,6 +1945,9 @@ struct host*		host;
 		if ( conns[i] != connSuccess )
 			rx_DestroyConnection(conns[i] );
 
+	free(addr);
+	free(conns);
+
 	if ( connSuccess ) return 0;	/* success */
 		else return 1;		/* failure */
 }
